#!/usr/bin/env python3

import hashlib, pyperclip, readline, gettext, pwinput
from Cryptodome.Cipher import AES
from Cryptodome.Random import get_random_bytes


t = gettext.translation(
    'sekreto',
    '/usr/share/locale/',
    fallback=True,
)

_ = t.gettext


IV_LENGTH = 12
KEY_LENGTH = 32
TAG_LENGTH = 16
SALT_LENGTH = 16
HASH_NAME = "SHA1"
ITERATION_COUNT = 65535


zwc = ["\u2060", "\u200B", "\u200D", "\u200E",
       "\u200F", "\u200C", "\u2061", "\u180E",
       "\u202A", "\u202C", "\u202D", "\u2062",
       "\u2063", "\u2064", "\u2065", "\u2066"]


def hid2hex(hid_string):
    mappings = {zwc[i]: hex(i)[2:].upper() for i in range(16)}
    hex_string = "".join(mappings.get(char, "") for char in hid_string)
    return hex_string


def hex2hid(hex_string):
    mappings = {hex(i)[2:].upper(): zwc[i] for i in range(16)}
    hid_string = "".join(mappings.get(char.upper(), "") for char in hex_string)
    return hid_string


def wrap(text):
    wrapped_text = "\uFEFF" + text + "\uFEFF"
    return wrapped_text


def unwrap(text):
    unwrapped_text = text.strip('\uFEFF')
    return unwrapped_text


def get_secret_key(password, salt):
    return hashlib.pbkdf2_hmac(
        HASH_NAME, password.encode(), salt, ITERATION_COUNT, KEY_LENGTH
    )


def encrypt(password, plain_message):
    salt = get_random_bytes(SALT_LENGTH)
    iv = get_random_bytes(IV_LENGTH)

    secret = get_secret_key(password, salt)

    cipher = AES.new(secret, AES.MODE_GCM, iv)

    encrypted_message_byte, tag = cipher.encrypt_and_digest(plain_message.encode("utf-8"))
    cipher_byte = salt + iv + encrypted_message_byte + tag

    encoded_cipher_byte = cipher_byte.hex()
    return encoded_cipher_byte


def decrypt(password, cipher_message):
    decoded_cipher_byte = bytes.fromhex(cipher_message)

    salt = decoded_cipher_byte[:SALT_LENGTH]
    iv = decoded_cipher_byte[SALT_LENGTH : (SALT_LENGTH + IV_LENGTH)]
    encrypted_message_byte = decoded_cipher_byte[(IV_LENGTH + SALT_LENGTH) : -TAG_LENGTH]
    tag = decoded_cipher_byte[-TAG_LENGTH:]
    secret = get_secret_key(password, salt)
    cipher = AES.new(secret, AES.MODE_GCM, iv)

    decrypted_message_byte = cipher.decrypt_and_verify(encrypted_message_byte, tag)
    return decrypted_message_byte.decode("utf-8")


while True:

    print()
    print(_("    [1]: Kaŝi mesaĝon en teksto"))
    print(_("    [2]: Montri kaŝitan mesaĝon"))
    print(_("    [0]: Eliri"))
    print()

    choice = input("Sekreto:~$ ")

    if choice == "1":
        secret_msg = input(_("Sekreta mesaĝo: "))
        secret_key = pwinput.pwinput(prompt =_("Pasvorto: "), mask="*")
        public_msg = input(_("Publika teksto: "))

        if len(public_msg) < 2:
            print(_("La longo de publika teksto devas esti almenaŭ du signoj!"))
            continue

        encrypt_msg = encrypt(secret_key, secret_msg)

        hiding = hex2hid(encrypt_msg)
        wrapping = wrap(hiding)

        result = public_msg[:len(public_msg) // 2] + wrapping + public_msg[len(public_msg) // 2:]

        try:
            pyperclip.copy(result)
            print(_("Mesaĝo kaŝita kaj kopiita!"))
        except pyperclip.PyperclipException:
            print(_("Eraro okazis dum kopiado!"))

    elif choice == "2":
        result = input(_("Enigu tekston kun mesaĝo: "))
        secret_key = pwinput.pwinput(prompt =_("Pasvorto: "), mask="*")

        extraction = unwrap(result)
        revealing = hid2hex(extraction)

        try:
            secret = decrypt(secret_key, revealing)
            print(_("Sekreta mesaĝo: "), secret)
        except:
            print(_("Vi enigis malĝustan teksto aŭ pasvorton!"))

    elif choice == "0":
        quit()
